package cz.jzitnik.game.threads;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.annotations.ThreadRegistry;
import cz.jzitnik.game.entities.Player;
import cz.jzitnik.tui.ScreenRenderer;

import org.jline.terminal.Terminal;
import org.reflections.Reflections;

public class ThreadProvider {
    private final Game game;
    private final ScreenRenderer screenRenderer;
    private final Terminal terminal;
    private final boolean[] isRunning;
    private final List<Thread> list = new ArrayList<>();

    public void start() {
        for (Thread thread : list) {
            thread.start();
        }
    }

    public ThreadProvider(Game game, ScreenRenderer screenRenderer, Terminal terminal, boolean[] isRunning) {
        this.game = game;
        this.screenRenderer = screenRenderer;
        this.terminal = terminal;
        this.isRunning = isRunning;
        registerHandlers();
    }

    private void registerHandlers() {
        Reflections reflections = new Reflections("cz.jzitnik.game.threads");
        Set<Class<?>> handlerClasses = reflections.getTypesAnnotatedWith(ThreadRegistry.class);

        for (Class<?> clazz : handlerClasses) {
            if (Thread.class.isAssignableFrom(clazz)) {
                try {
                    for (var constructor : clazz.getDeclaredConstructors()) {
                        var paramTypes = constructor.getParameterTypes();
                        var params = new Object[paramTypes.length];
                        boolean suitable = true;

                        for (int i = 0; i < paramTypes.length; i++) {
                            Class<?> type = paramTypes[i];
                            if (type == Game.class) params[i] = game;
                            else if (type == ScreenRenderer.class) params[i] = screenRenderer;
                            else if (type == Terminal.class) params[i] = terminal;
                            else if (type == boolean[].class) params[i] = isRunning;
                            else if (type == Player.class) params[i] = game.getPlayer();
                            else {
                                suitable = false;
                                break;
                            }
                        }

                        if (suitable) {
                            constructor.setAccessible(true);
                            Thread instance = (Thread) constructor.newInstance(params);
                            list.add(instance);
                            break; // Found a matching constructor, go to next class
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
